// maf_order.c -- order rows in a maf-file entries according to a given list

#include "lib.h"
#include "maf.h"
#include "mz_util.h"

int main(int argc, char **argv) {
	struct mafFile *mf;
	struct mafAli *a;
	struct mafComp *c, *last, **location;
	char **species;
	int i, nspecies, head = 1;;

	if (argc < 3)
		fatal("args: maf-file species1 species2 .. [nohead]");
	if (same_string(argv[argc-1], "nohead")) {
		head = 0;
		--argc;
	}
	
	mf = mafOpen(argv[1],1);
	nspecies = argc - 2;
	species = argv + 2;
	if (head)
		mafWriteStart(stdout, "maf_project");
	location = ckalloc(nspecies*sizeof(struct mafComp *));
	while ((a = mafNext(mf)) != NULL) {
		for (i = 0; i < nspecies; ++i)
			location[i] = NULL;
		for (c = a->components; c != NULL; c = c->next) {
			for (i = 0; i < nspecies; ++i)
				if (same_string(c->src, species[i]))
					break;
			if (i == nspecies)
				fatalf("unnamed species `%s'", c->src);
			if (location[i] != NULL) {
				mafWrite(stderr, a);
				fatalf("species `%s'named more than once",
				  c->src);
			}
			location[i] = c;
		}

		for (last = NULL, i = 0; i < nspecies; ++i)
			if (location[i] != NULL) {
				if (last == NULL)
					a->components = location[i];
				else
					last->next = location[i];
				last = location[i];
			}
		if (last == NULL)
			fatal("maf_order: cannot happen");
		last->next = NULL;
		mafWrite(stdout, a);
		mafAliFree(&a);
	}
	free(location);
	return 0;
}
