// seq2tba.c -- convert a sequence to a simple TBA in .maf format, divided
// according to a pairwise alignment
#include "util.h"
#include "maf.h"
#include "seq.h"

static void piece2maf(struct mafAli *a, uchar *s, int start, int beyond) {
	struct mafComp *c = a->components;
	int i, size = beyond - start;

	if (start < beyond) {
		a->textSize = size;
		c->start = start;
		c->size = size;
		c->text = ckalloc((size+1)*sizeof(char));
		for (i = 0; i < size; ++i)
			c->text[i] = s[start+i];
		c->text[size] = '\0';
		mafWrite(stdout, a);
		free(c->text);
	}
}

int main(int argc, char **argv) {
	SEQ *sf;
	struct mafFile *mf;
	struct mafAli A, *a, *b;
	struct mafComp C, *c, *d;
	int st, old_beyond;

	argv0 = "seq2tba";
	if (argc != 3)
		fatal("args: seqFile mafFile");
	
	sf = seq_get(argv[1]);
	
	mafWriteStart(stdout, "seq2tba");
	
	a = &A;
	c = &C;
	a->components = c;
	a->score = 0.0;
	c->srcSize = SEQ_LEN(sf);
	a->next = NULL;
	c->src = copy_string(argv[1]);
	c->strand = '+';
	c->next = NULL;

	mf = mafOpen(argv[2], 1);
	old_beyond = 0;
	while ((b = mafNext(mf)) != NULL) {
		d = b->components;
		if ((st = d->start) < old_beyond)
			fatalf("overlapping maf entries: %d and %d",
			  d->start, old_beyond);
		piece2maf(a, SEQ_CHARS(sf), old_beyond, st);
		piece2maf(a, SEQ_CHARS(sf), st, (old_beyond = st + d->size));
	}
	piece2maf(a, SEQ_CHARS(sf), old_beyond, SEQ_LEN(sf));

	return 0;
}
